@@ -14,7 +14,6 @@
 (**************************************************************************)
 
 open Misc
-open Compenv
 
 type info = {
   source_file : string;
@@ -33,7 +32,7 @@ let annot i = i.output_prefix ^ ".annot"
 
 let with_info ~native ~tool_name ~source_file ~output_prefix ~dump_ext k =
   Compmisc.init_path ();
-  let module_name = module_of_filename source_file output_prefix in
+  let module_name = Compenv.module_of_filename source_file output_prefix in
   Env.set_unit_name module_name;
   let env = Compmisc.initial_env() in
   let dump_file = String.concat "." [output_prefix; dump_ext] in
@@ -107,6 +106,8 @@ let typecheck_impl i parsetree =
        i.source_file i.output_prefix i.module_name i.env)
   |> print_if i.ppf_dump Clflags.dump_typedtree
     Printtyped.implementation_with_coercion
+  |> print_if i.ppf_dump Clflags.dump_shape
+    (fun fmt {Typedtree.shape; _} -> Shape.print fmt shape)
 
 let implementation info ~backend =
   Profile.record_call info.source_file @@ fun () ->
