@@ -140,7 +140,6 @@ end = struct
     { subst : Clambda.ulambda Variable.Map.t;
       var : V.t Variable.Map.t;
       mutable_var : V.t Mutable_variable.Map.t;
-      toplevel : bool;
       allocated_constant_for_symbol : Allocated_const.t Symbol.Map.t;
     }
 
@@ -148,7 +147,6 @@ end = struct
     { subst = Variable.Map.empty;
       var = Variable.Map.empty;
       mutable_var = Mutable_variable.Map.empty;
-      toplevel = false;
       allocated_constant_for_symbol = Symbol.Map.empty;
     }
 
@@ -430,7 +428,7 @@ and to_clambda_switch t env cases num_keys default =
   List.iter
     (fun (key, lam) ->
       index.(key) <- store.act_store () lam;
-      smallest_key := min key !smallest_key
+      smallest_key := Int.min key !smallest_key
     )
     cases;
   if !smallest_key < num_keys then begin
@@ -547,6 +545,7 @@ and to_clambda_set_of_closures t env
       body = to_clambda t env_body function_decl.body;
       dbg = function_decl.dbg;
       env = Some env_var;
+      poll = function_decl.poll;
     }
   in
   let funs = List.map to_clambda_function all_functions in
@@ -592,6 +591,7 @@ and to_clambda_closed_set_of_closures t env symbol
       body;
       dbg = function_decl.dbg;
       env = None;
+      poll = function_decl.poll;
     }
   in
   let ufunct = List.map to_clambda_function functions in
