@@ -72,12 +72,10 @@ let update_value_parameters_text v =
    [parameter_list_from_arrows t = [ a ; b ]] if t = a -> b -> c.*)
 let parameter_list_from_arrows typ =
   let rec iter t =
-    match t.Types.desc with
+    match Types.get_desc t with
       Types.Tarrow (l, t1, t2, _) ->
         (l, t1) :: (iter t2)
     | Types.Tlink texp
-    | Types.Tsubst texp ->
-        iter texp
     | Types.Tpoly (texp, _) -> iter texp
     | Types.Tvar _
     | Types.Ttuple _
@@ -89,6 +87,8 @@ let parameter_list_from_arrows typ =
     | Types.Tpackage _
     | Types.Tvariant _ ->
         []
+    | Types.Tsubst _ ->
+        assert false
   in
   iter typ
 
@@ -99,10 +99,9 @@ let parameter_list_from_arrows typ =
    parameter names from the .ml and the type from the .mli file. *)
 let dummy_parameter_list typ =
   let normal_name = Odoc_misc.label_name in
-  Printtyp.mark_loops typ;
   let liste_param = parameter_list_from_arrows typ in
   let rec iter (label, t) =
-    match t.Types.desc with
+    match Types.get_desc t with
     | Types.Ttuple l ->
         let open Asttypes in
         if label = Nolabel then
@@ -114,10 +113,10 @@ let dummy_parameter_list typ =
             { Odoc_parameter.sn_name = normal_name label ;
               Odoc_parameter.sn_type = t ;
               Odoc_parameter.sn_text = None }
-    | Types.Tlink t2
-    | Types.Tsubst t2 ->
+    | Types.Tlink t2 ->
         (iter (label, t2))
-
+    | Types.Tsubst _ ->
+        assert false
     | _ ->
         Odoc_parameter.Simple_name
           { Odoc_parameter.sn_name = normal_name label ;
@@ -129,7 +128,7 @@ let dummy_parameter_list typ =
 (** Return true if the value is a function, i.e. has a functional type.*)
 let is_function v =
   let rec f t =
-    match t.Types.desc with
+    match Types.get_desc t with
       Types.Tarrow _ ->
         true
     | Types.Tlink t ->
