@@ -29,7 +29,7 @@ CAMLprim value unix_mkfifo(value path, value mode)
   char * p;
   int ret;
   caml_unix_check_path(path, "mkfifo");
-  p = caml_strdup(String_val(path));
+  p = caml_stat_strdup(String_val(path));
   caml_enter_blocking_section();
   ret = mkfifo(p, Int_val(mode));
   caml_leave_blocking_section();
@@ -52,7 +52,7 @@ CAMLprim value unix_mkfifo(value path, value mode)
   char * p;
   int ret;
   caml_unix_check_path(path, "mkfifo");
-  p = caml_strdup(String_val(path));
+  p = caml_stat_strdup(String_val(path));
   caml_enter_blocking_section();
   ret = mknod(p, (Int_val(mode) & 07777) | S_IFIFO, 0);
   caml_leave_blocking_section();
