@@ -23,6 +23,7 @@
 #include "unixsupport.h"
 #include <errno.h>
 #include <winioctl.h>
+#include <caml/winsupport.h>
 
 CAMLprim value unix_readlink(value opath)
 {
@@ -83,7 +84,7 @@ CAMLprim value unix_readlink(value opath)
           win_wide_char_to_multi_byte(
             point->SymbolicLinkReparseBuffer.PathBuffer + point->SymbolicLinkReparseBuffer.SubstituteNameOffset / sizeof(WCHAR),
             cbLen,
-            String_val(result),
+            (char *)String_val(result),
             len);
           CloseHandle(h);
         }
