@@ -25,9 +25,6 @@
 #include "caml/mlvalues.h"
 #include "caml/roots.h"
 #include "caml/signals.h"
-#if defined(NATIVE_CODE) && defined(WITH_SPACETIME)
-#include "caml/spacetime.h"
-#endif
 
 struct final {
   value fun;
@@ -170,9 +167,6 @@ value caml_final_do_calls_exn (void)
 {
   struct final f;
   value res;
-#if defined(NATIVE_CODE) && defined(WITH_SPACETIME)
-  void* saved_spacetime_trie_node_ptr;
-#endif
 
   if (!running_finalisation_function && to_do_hd != NULL){
     if (caml_finalise_begin_hook != NULL) (*caml_finalise_begin_hook) ();
@@ -189,17 +183,7 @@ value caml_final_do_calls_exn (void)
       -- to_do_hd->size;
       f = to_do_hd->item[to_do_hd->size];
       running_finalisation_function = 1;
-#if defined(NATIVE_CODE) && defined(WITH_SPACETIME)
-      /* We record the finaliser's execution separately.
-         (The code of [caml_callback_exn] will do the hard work of finding
-         the correct place in the trie.) */
-      saved_spacetime_trie_node_ptr = caml_spacetime_trie_node_ptr;
-      caml_spacetime_trie_node_ptr = caml_spacetime_finaliser_trie_root;
-#endif
       res = caml_callback_exn (f.fun, f.val + f.offset);
-#if defined(NATIVE_CODE) && defined(WITH_SPACETIME)
-      caml_spacetime_trie_node_ptr = saved_spacetime_trie_node_ptr;
-#endif
       running_finalisation_function = 0;
       if (Is_exception_result (res)) return res;
     }
