@@ -210,7 +210,7 @@ val kind_size_in_bytes : ('a, 'b) kind -> int
 (** {1 Array layouts} *)
 
 type c_layout = C_layout_typ (**)
-(** See {!Bigarray.fortran_layout}.*)
+(** See {!type:Bigarray.fortran_layout}.*)
 
 type fortran_layout = Fortran_layout_typ (**)
 (** To facilitate interoperability with existing C and Fortran code,
@@ -233,10 +233,10 @@ type fortran_layout = Fortran_layout_typ (**)
    and [(x+1, y)] are adjacent in memory.
 
    Each layout style is identified at the type level by the
-   phantom types {!Bigarray.c_layout} and {!Bigarray.fortran_layout}
+   phantom types {!type:Bigarray.c_layout} and {!type:Bigarray.fortran_layout}
    respectively. *)
 
-(** {7 Supported layouts}
+(** {2 Supported layouts}
 
    The GADT type ['a layout] represents one of the two supported
    memory layouts: C-style or Fortran-style. Its constructors are
@@ -255,7 +255,7 @@ val fortran_layout : fortran_layout layout
 
 module Genarray :
   sig
-  type ('a, 'b, 'c) t
+  type (!'a, !'b, !'c) t
   (** The type [Genarray.t] is the type of Bigarrays with variable
      numbers of dimensions.  Any number of dimensions between 0 and 16
      is supported.
@@ -298,6 +298,34 @@ module Genarray :
      is not in the range 0 to 16 inclusive, or if one of the dimensions
      is negative. *)
 
+  val init: ('a, 'b) kind -> 'c layout -> int array -> (int array -> 'a) ->
+            ('a, 'b, 'c) t
+  (** [Genarray.init kind layout dimensions f] returns a new Bigarray [b]
+      whose element kind is determined by the parameter [kind] (one of
+      [float32], [float64], [int8_signed], etc) and whose layout is
+      determined by the parameter [layout] (one of [c_layout] or
+      [fortran_layout]).  The [dimensions] parameter is an array of
+      integers that indicate the size of the Bigarray in each dimension.
+      The length of [dimensions] determines the number of dimensions
+      of the Bigarray.
+
+      Each element [Genarray.get b i] is initialized to the result of [f i].
+      In other words, [Genarray.init kind layout dimensions f] tabulates
+      the results of [f] applied to the indices of a new Bigarray whose
+      layout is described by [kind], [layout] and [dimensions].  The index
+      array [i] may be shared and mutated between calls to f.
+
+      For instance, [Genarray.init int c_layout [|2; 1; 3|]
+      (Array.fold_left (+) 0)] returns a fresh Bigarray of integers, in C
+      layout, having three dimensions (2, 1, 3, respectively), with the
+      element values 0, 1, 2, 1, 2, 3.
+
+      [Genarray.init] raises [Invalid_argument] if the number of dimensions
+      is not in the range 0 to 16 inclusive, or if one of the dimensions
+      is negative.
+
+      @since 4.12.0 *)
+
   external num_dims: ('a, 'b, 'c) t -> int = "caml_ba_num_dims"
   (** Return the number of dimensions of the given Bigarray. *)
 
@@ -477,7 +505,7 @@ module Genarray :
    faster operations, and more precise static type-checking.
    @since 4.05.0 *)
 module Array0 : sig
-  type ('a, 'b, 'c) t
+  type (!'a, !'b, !'c) t
   (** The type of zero-dimensional Bigarrays whose elements have
      OCaml type ['a], representation kind ['b], and memory layout ['c]. *)
 
@@ -486,6 +514,12 @@ module Array0 : sig
      [kind] and [layout] determine the array element kind and the array
      layout as described for {!Genarray.create}. *)
 
+  val init: ('a, 'b) kind -> 'c layout -> 'a -> ('a, 'b, 'c) t
+  (** [Array0.init kind layout v] behaves like [Array0.create kind layout]
+     except that the element is additionally initialized to the value [v].
+
+     @since 4.12.0 *)
+
   external kind: ('a, 'b, 'c) t -> ('a, 'b) kind = "caml_ba_kind"
   (** Return the kind of the given Bigarray. *)
 
@@ -535,7 +569,7 @@ end
    Statically knowing the number of dimensions of the array allows
    faster operations, and more precise static type-checking. *)
 module Array1 : sig
-  type ('a, 'b, 'c) t
+  type (!'a, !'b, !'c) t
   (** The type of one-dimensional Bigarrays whose elements have
      OCaml type ['a], representation kind ['b], and memory layout ['c]. *)
 
@@ -545,6 +579,22 @@ module Array1 : sig
      determine the array element kind and the array layout
      as described for {!Genarray.create}. *)
 
+  val init: ('a, 'b) kind -> 'c layout -> int -> (int -> 'a) ->
+            ('a, 'b, 'c) t
+  (** [Array1.init kind layout dim f] returns a new Bigarray [b]
+     of one dimension, whose size is [dim].  [kind] and [layout]
+     determine the array element kind and the array layout
+     as described for {!Genarray.create}.
+
+     Each element [Array1.get b i] of the array is initialized to the
+     result of [f i].
+
+     In other words, [Array1.init kind layout dimensions f] tabulates
+     the results of [f] applied to the indices of a new Bigarray whose
+     layout is described by [kind], [layout] and [dim].
+
+     @since 4.12.0 *)
+
   external dim: ('a, 'b, 'c) t -> int = "%caml_ba_dim_1"
   (** Return the size (dimension) of the given one-dimensional
      Bigarray. *)
@@ -632,17 +682,34 @@ end
    case of two-dimensional arrays. *)
 module Array2 :
   sig
-  type ('a, 'b, 'c) t
+  type (!'a, !'b, !'c) t
   (** The type of two-dimensional Bigarrays whose elements have
      OCaml type ['a], representation kind ['b], and memory layout ['c]. *)
 
   val create: ('a, 'b) kind ->  'c layout -> int -> int -> ('a, 'b, 'c) t
   (** [Array2.create kind layout dim1 dim2] returns a new Bigarray of
-     two dimension, whose size is [dim1] in the first dimension
+     two dimensions, whose size is [dim1] in the first dimension
      and [dim2] in the second dimension.  [kind] and [layout]
      determine the array element kind and the array layout
      as described for {!Bigarray.Genarray.create}. *)
 
+  val init: ('a, 'b) kind ->  'c layout -> int -> int ->
+            (int -> int -> 'a) -> ('a, 'b, 'c) t
+  (** [Array2.init kind layout dim1 dim2 f] returns a new Bigarray [b]
+     of two dimensions, whose size is [dim2] in the first dimension
+     and [dim2] in the second dimension.  [kind] and [layout]
+     determine the array element kind and the array layout
+     as described for {!Bigarray.Genarray.create}.
+
+     Each element [Array2.get b i j] of the array is initialized to
+     the result of [f i j].
+
+     In other words, [Array2.init kind layout dim1 dim2 f] tabulates
+     the results of [f] applied to the indices of a new Bigarray whose
+     layout is described by [kind], [layout], [dim1] and [dim2].
+
+     @since 4.12.0 *)
+
   external dim1: ('a, 'b, 'c) t -> int = "%caml_ba_dim_1"
   (** Return the first dimension of the given two-dimensional Bigarray. *)
 
@@ -748,17 +815,34 @@ end
    of three-dimensional arrays. *)
 module Array3 :
   sig
-  type ('a, 'b, 'c) t
+  type (!'a, !'b, !'c) t
   (** The type of three-dimensional Bigarrays whose elements have
      OCaml type ['a], representation kind ['b], and memory layout ['c]. *)
 
   val create: ('a, 'b) kind -> 'c layout -> int -> int -> int -> ('a, 'b, 'c) t
   (** [Array3.create kind layout dim1 dim2 dim3] returns a new Bigarray of
-     three dimension, whose size is [dim1] in the first dimension,
+     three dimensions, whose size is [dim1] in the first dimension,
      [dim2] in the second dimension, and [dim3] in the third.
      [kind] and [layout] determine the array element kind and
      the array layout as described for {!Bigarray.Genarray.create}. *)
 
+  val init: ('a, 'b) kind ->  'c layout -> int -> int -> int ->
+            (int -> int -> int -> 'a) -> ('a, 'b, 'c) t
+  (** [Array3.init kind layout dim1 dim2 dim3 f] returns a new Bigarray [b]
+     of three dimensions, whose size is [dim1] in the first dimension,
+     [dim2] in the second dimension, and [dim3] in the third.
+     [kind] and [layout] determine the array element kind and the array
+     layout as described for {!Bigarray.Genarray.create}.
+
+     Each element [Array3.get b i j k] of the array is initialized to
+     the result of [f i j k].
+
+     In other words, [Array3.init kind layout dim1 dim2 dim3 f] tabulates
+     the results of [f] applied to the indices of a new Bigarray whose
+     layout is described by [kind], [layout], [dim1], [dim2] and [dim3].
+
+     @since 4.12.0 *)
+
   external dim1: ('a, 'b, 'c) t -> int = "%caml_ba_dim_1"
   (** Return the first dimension of the given three-dimensional Bigarray. *)
 
@@ -887,7 +971,8 @@ end
 external genarray_of_array0 :
   ('a, 'b, 'c) Array0.t -> ('a, 'b, 'c) Genarray.t = "%identity"
 (** Return the generic Bigarray corresponding to the given zero-dimensional
-   Bigarray. @since 4.05.0 *)
+    Bigarray.
+    @since 4.05.0 *)
 
 external genarray_of_array1 :
   ('a, 'b, 'c) Array1.t -> ('a, 'b, 'c) Genarray.t = "%identity"
