@@ -3,11 +3,9 @@
    ocamlrunparam += ",b=1"
    * bytecode
    * native
-     compare_programs = "false"
    * native
      ocamlopt_flags = "-O3"
      compiler_directory_suffix = ".O3"
-     compare_programs = "false"
 *)
 
 (* A test for inlined stack backtraces *)
@@ -25,5 +23,4 @@ let i x =
   if h x = () then ()
 
 let () =
-  Printexc.record_backtrace true;
   i ()
