@@ -1,6 +1,4 @@
-(* TEST
-   compare_programs = "false" (* See https://github.com/ocaml/ocaml/pull/8853 *)
-*)
+(* TEST *)
 
 let current = ref 0;;
 
