@@ -1,3 +1,13 @@
+(* TEST
+   unset FOO
+   * hasunix
+   include unix
+   script = "sh ${test_source_directory}/has-execvpe.sh"
+   ** script
+   *** bytecode
+   *** native
+*)
+
 open Printf
 
 let _ =
