@@ -1,3 +1,7 @@
+(* TEST
+   flags += " -unsafe "
+*)
+
 (* Good test for loops. Best compiled with -unsafe. *)
 
 let rec qsort lo hi (a : int array) =
