@@ -1,17 +1,14 @@
 (* TEST
 flags = "-no-alias-deps"
 compile_only = "true"
-files = "c.mli d.mli"
+readonly_files = "b.cmi c.mli d.mli"
 * setup-ocamlc.byte-build-env
-** script
-script =
-  "cp ${test_source_directory}/b.cmi.invalid ${test_build_directory}/b.cmi"
-*** ocamlc.byte
+** ocamlc.byte
 all_modules = "c.mli d.mli aliases.ml"
-**** check-ocamlc.byte-output
-***** ocamlobjinfo
+*** check-ocamlc.byte-output
+**** ocamlobjinfo
 program = "aliases.cmo"
-****** check-program-output
+***** check-program-output
 *)
 
 module A' = A (* missing a.cmi *)
