@@ -6,17 +6,19 @@ type 'a x = private [> `x] as 'a;;
 [%%expect {|
 Line 1:
 Error: Type declarations do not match:
-         type 'a x = private [> `x ] constraint 'a = 'a x
+         type !'a x = private [> `x ] constraint 'a = 'a x
        is not included in
          type 'a x
-       Their constraints differ.
+       Their parameters differ
+       The type 'b x as 'b is not equal to the type 'a
 |}, Principal{|
 Line 1:
 Error: Type declarations do not match:
-         type 'a x = private 'a constraint 'a = [> `x ]
+         type !'a x = private 'a constraint 'a = [> `x ]
        is not included in
          type 'a x
-       Their constraints differ.
+       Their parameters differ
+       The type [> `x ] is not equal to the type 'a
 |}];;
 
 
