@@ -1,6 +1,6 @@
 (* TEST
 include tool-ocaml-lib
-flags = "-w a"
+flags = "-w -a"
 ocaml_script_as_argument = "true"
 * setup-ocaml-build-env
 ** ocaml
