@@ -1,3 +1,11 @@
+(* TEST
+include tool-ocaml-lib
+flags = "-w -a"
+ocaml_script_as_argument = "true"
+* setup-ocaml-build-env
+** ocaml
+*)
+
 open Lib;;
 let x = Array.make 20 "" in
 let w = weak_create 20 in
