@@ -1,7 +1,7 @@
 (* TEST
 * setup-ocamlc.byte-build-env
 ** ocamlc.byte
-    flags = "-stop-after typing -dtypedtree"
+    flags = "-stop-after typing -dno-unique-ids -dtypedtree"
     ocamlc_byte_exit_status = "0"
 *** check-ocamlc.byte-output
 *)
