@@ -17,7 +17,7 @@ let g (Aux(Second, f)) = f it;;
 [%%expect{|
 type 'a t = 'a constraint 'a = [< `Bar | `Foo ]
 type 'a s = 'a constraint 'a = [< `Bar | `Baz | `Foo > `Bar ]
-type 'a first = First : 'b t second -> ([< `Bar | `Foo ] as 'b) t first
+type 'a first = First : 'a t second -> ([< `Bar | `Foo ] as 'a) t first
 and 'a second = Second : [< `Bar | `Baz | `Foo > `Bar ] s second
 type aux = Aux : ([< `Bar | `Foo ] as 'a) t second * ('a -> int) -> aux
 val it : [< `Bar | `Foo > `Bar ] = `Bar
@@ -26,6 +26,6 @@ Line 11, characters 27-29:
                                 ^^
 Error: This expression has type [< `Bar | `Foo > `Bar ]
        but an expression was expected of type [< `Bar | `Foo ]
-       The second variant type is bound to $Aux,
+       The second variant type is bound to $Aux_'a,
        it may not allow the tag(s) `Bar
 |}];;
