@@ -10,7 +10,7 @@ end;;
 
 [%%expect{|
 type t = int
-Line 3, characters 0-31:
+Lines 3-5, characters 0-3:
 3 | struct
 4 |   type t = [`T of t]
 5 | end..
@@ -23,8 +23,11 @@ Error: Signature mismatch:
          type t = [ `T of t/2 ]
        is not included in
          type t = [ `T of t/1 ]
-       Line 1, characters 0-12:
-         Definition of type t/1
+       The type [ `T of t/1 ] is not equal to the type [ `T of t/2 ]
+       Type t/1 = [ `T of t/1 ] is not equal to type t/2 = int
+       Types for tag `T are incompatible
        Line 4, characters 2-20:
+         Definition of type t/1
+       Line 1, characters 0-12:
          Definition of type t/2
 |}]
