@@ -8,7 +8,7 @@ let () = (let module L = List in raise Exit); () ;;
 Line 1, characters 33-43:
 1 | let () = (let module L = List in raise Exit); () ;;
                                      ^^^^^^^^^^
-Warning 21: this statement never returns (or has an unsound type.)
+Warning 21 [nonreturning-statement]: this statement never returns (or has an unsound type.)
 Exception: Stdlib.Exit.
 |}]
 let () = (let exception E in raise Exit); ();;
@@ -16,7 +16,7 @@ let () = (let exception E in raise Exit); ();;
 Line 1, characters 29-39:
 1 | let () = (let exception E in raise Exit); ();;
                                  ^^^^^^^^^^
-Warning 21: this statement never returns (or has an unsound type.)
+Warning 21 [nonreturning-statement]: this statement never returns (or has an unsound type.)
 Exception: Stdlib.Exit.
 |}]
 let () = (raise Exit : _); ();;
@@ -24,7 +24,7 @@ let () = (raise Exit : _); ();;
 Line 1, characters 10-20:
 1 | let () = (raise Exit : _); ();;
               ^^^^^^^^^^
-Warning 21: this statement never returns (or has an unsound type.)
+Warning 21 [nonreturning-statement]: this statement never returns (or has an unsound type.)
 Exception: Stdlib.Exit.
 |}]
 let () = (let open Stdlib in raise Exit); ();;
@@ -32,6 +32,6 @@ let () = (let open Stdlib in raise Exit); ();;
 Line 1, characters 29-39:
 1 | let () = (let open Stdlib in raise Exit); ();;
                                  ^^^^^^^^^^
-Warning 21: this statement never returns (or has an unsound type.)
+Warning 21 [nonreturning-statement]: this statement never returns (or has an unsound type.)
 Exception: Stdlib.Exit.
 |}]
