@@ -1,5 +1,5 @@
 (* TEST
-   flags = " -w A -strict-sequence "
+   flags = " -w +A -strict-sequence "
    * expect
 *)
 
@@ -17,7 +17,7 @@ let f : label choice -> bool = function Left -> true;; (* warn *)
 Line 1, characters 31-52:
 1 | let f : label choice -> bool = function Left -> true;; (* warn *)
                                    ^^^^^^^^^^^^^^^^^^^^^
-Warning 8: this pattern-matching is not exhaustive.
+Warning 8 [partial-match]: this pattern-matching is not exhaustive.
 Here is an example of a case that is not matched:
 Right
 val f : CamlinternalOO.label choice -> bool = <fun>
