@@ -1,5 +1,5 @@
 (* TEST
-   flags = " -w A -strict-sequence "
+   flags = " -w +A -strict-sequence "
    * expect
 *)
 
@@ -36,4 +36,7 @@ Error: Signature mismatch:
          val f : fpclass -> Stdlib.fpclass
        is not included in
          val f : fpclass -> fpclass
+       The type fpclass -> Stdlib.fpclass is not compatible with the type
+         fpclass -> fpclass
+       Type Stdlib.fpclass is not compatible with type fpclass
 |}]
