@@ -1,5 +1,5 @@
 (* TEST
-   flags = " -w A "
+   flags = " -w +A "
    * expect
 *)
 
@@ -8,7 +8,7 @@ module Foo(Unused : sig end) = struct end;;
 Line 1, characters 11-17:
 1 | module Foo(Unused : sig end) = struct end;;
                ^^^^^^
-Warning 60: unused module Unused.
+Warning 60 [unused-module]: unused module Unused.
 module Foo : functor (Unused : sig end) -> sig end
 |}]
 
@@ -17,7 +17,7 @@ module type S = functor (Unused : sig end) -> sig end;;
 Line 1, characters 25-31:
 1 | module type S = functor (Unused : sig end) -> sig end;;
                              ^^^^^^
-Warning 67: unused functor parameter Unused.
+Warning 67 [unused-functor-parameter]: unused functor parameter Unused.
 module type S = functor (Unused : sig end) -> sig end
 |}]
 
@@ -28,6 +28,6 @@ end;;
 Line 2, characters 12-18:
 2 |   module M (Unused : sig end) : sig end
                 ^^^^^^
-Warning 67: unused functor parameter Unused.
+Warning 67 [unused-functor-parameter]: unused functor parameter Unused.
 module type S = sig module M : functor (Unused : sig end) -> sig end end
 |}]
