@@ -9,7 +9,7 @@ let rec f () = 3;;
 Line 3, characters 8-9:
 3 | let rec f () = 3;;
             ^
-Warning 39: unused rec flag.
+Warning 39 [unused-rec-flag]: unused rec flag.
 val f : unit -> int = <fun>
 |}];;
 
@@ -23,7 +23,7 @@ let[@warning "+39"] rec h () = 3;;
 Line 1, characters 24-25:
 1 | let[@warning "+39"] rec h () = 3;;
                             ^
-Warning 39: unused rec flag.
+Warning 39 [unused-rec-flag]: unused rec flag.
 val h : unit -> int = <fun>
 |}];;
 
@@ -44,6 +44,6 @@ let[@warning "+39"] rec h () = 3;;
 Line 1, characters 24-25:
 1 | let[@warning "+39"] rec h () = 3;;
                             ^
-Warning 39: unused rec flag.
+Warning 39 [unused-rec-flag]: unused rec flag.
 val h : unit -> int = <fun>
 |}];;
